<?php
require_once(dirname(__FILE__) . '/constants.php');
require_once(dirname(__FILE__) . '/session.php');
require_once(dirname(__FILE__) . '/functions.php');
require_once 'MDB2.php';
PEAR::setErrorHandling(PEAR_ERROR_RETURN);

/**
 * getPEARErrorInfo - Returns full info from PEAR::Error
 */
if (!function_exists('getPEARErrorInfo')) {
	function getPEARErrorInfo($dberrobj, $sep = "\n") {
		if (PEAR::isError($dberrobj)) {
			$dbmsg = $dberrobj->getUserInfo();
			$dbmsgd = $dberrobj->getDebugInfo();
			$dbmsgd = (strcmp($dbmsg, $dbmsgd) == 0) ? '' : $sep . "dbg: " . $dbmsgd;
			return "msg: " . $dberrobj->getMessage() . "(" . $dberrobj->getCode() . ")" . $sep
				   . "dbgmsg: " . $dbmsg . $dbmsgd;
		} else {
			return "";
		}
	}
}

class TMDB {
	public $connection;
	public $db_ok;

	function TMDB() {
		global $logger;

		$dsn = DB_ENGINE . '://' . DB_USER . ':' . DB_PASS . '@' . DB_HOST . '/' . DB_FILE;
		$this->connection =& MDB2::factory($dsn, array('debug' => 0));
		if (PEAR::isError($this->connection)) {
			$this->db_ok = false;
			$logger->log("DB Fatal Error:\n" . getPEARErrorInfo($this->connection), PEAR_LOG_CRIT);
			return;
		}
		$this->db_ok = true;

		$this->connection->setOption('field_case', CASE_LOWER);
		$this->connection->setOption('portability', MDB2_PORTABILITY_NUMROWS | MDB2_PORTABILITY_FIX_CASE);
		$this->connection->setFetchMode(MDB2_FETCHMODE_ASSOC);
		//$this->connection->setFetchMode(MDB2_FETCHMODE_OBJECT);
		$this->connection->LoadModule('Extended');
	}

	/**
	 * getQueryResultAsArray
	 *
	 * @param      $query
	 * @param bool $singleton
	 *
	 * @return array|null
	 */
	function getQueryResultAsArray($query, $singleton = false) {
		global $logger;

		$result = $this->connection->queryAll(trim($query));
		if (PEAR::isError($result)) {
			$logger->log("getQueryResultAsArray($query) ERR:\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
			return null;
		}
		//$logger->log("getQueryResultAsArray($query) RES:\n" . print_r($result, true), PEAR_LOG_DEBUG);

		/*if (!function_exists('to_utf8')) {
			function to_utf8(&$item, $key) {
				$item = iconv('cp1251', 'utf-8', $item);
			}
		}
		array_walk_recursive($result, 'to_utf8');*/

		if ($singleton === false)
			return $result;
		else {
			if (empty($result))
				return null;
			else
				return $result[0];
		}
	}

	/**
	 * query
	 *
	 * @param $query
	 *
	 * @return query
	 */
	function query($query) {
		return $this->connection->query($query);
	}

	/**
	 * prepare - wrapper
	 */
	function prepare($query, $types = null, $result_types = null) {
		global $logger;

		$result = $this->connection->prepare($query, $types, $result_types);
		if (PEAR::isError($result)) {
			$logger->log("prepare($query, $types) ERR:\n" . getPEARErrorInfo($result));
			return null;
		}
		return $result;
	}

	/**
	 * execute - wrapper
	 */
	function execute($preparedquery, $data = null) {
		global $logger;

		if (!$preparedquery) {
			$logger->log("execute ERR0 (called for wrong obj):\n" . print_r($preparedquery, true), PEAR_LOG_ERR);
			return null;
		}

		//$logger->log("execute Q:\n" . print_r($preparedquery, true));
		$result = $preparedquery->execute($data);

		if (PEAR::isError($result)) {
			$logger->log("execute ERR:\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
			return null;
		}
		//$logger->log("execute OK:\n" . print_r($result, true));
		if (!isset($preparedquery->is_manip) || !$preparedquery->is_manip) {
			$res = $result->fetchAll();
			$result->free();
		} else
			$res = $result;
		return $res;
	}

	/**
	 * register_login_session
	 */
	function register_login_session($login, $webid, $time) {
		global $logger;

		$q = "update " . TB_WEB_CACHE . " set acc_userid='$webid', last_time=$time where acc_pin='$login' and last_time>0";
		$pq = $this->prepare($q, null, MDB2_PREPARE_MANIP);
		$result = $this->execute($pq);
		if (PEAR::isError($result) || !is_numeric($result)) {
			$logger->log("database->register_login_session($q):ERR1=\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		}
		if ($result == 0) {
			$q = "insert into " . TB_WEB_CACHE . " (acc_id,acc_emitkcid,acc_pin,acc_userid,last_time) values (0, 0, '$login', '$webid', $time)";
			$pq = $this->prepare($q, null, MDB2_PREPARE_MANIP);
			$result = $this->execute($pq);
			if (PEAR::isError($result) || !is_numeric($result)) {
				$logger->log("database->register_login_session($q):ERR2=\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
				return false;
			} else
				$logger->log("database->register_login_session('$login'):OK=new", PEAR_LOG_DEBUG);
		} /*else
			$logger->log("database->register_login_session('$login'):OK=update", PEAR_LOG_DEBUG);*/
		return true;
	}

	/**
	 *  log_login_session
	 */
	function log_login_session($login, $userid, $time) {
		global $logger;

		$remote_addr = $_SERVER['REMOTE_ADDR'];
		$q = "insert into " . TB_WEB_LOG . " (acc_id,acc_emitkcid,acc_pin,lg_time,lg_host) values (0, 0, '$login', $time, '$remote_addr')";
		$logger->log("log_login_session(0, 0, '$login', $time, $remote_addr)", PEAR_LOG_DEBUG);
		return $this->connection->query($q);
	}


	/**
	 * get_account_by_webid
	 *
	 * @return array
	 */
	function get_account_by_webid($webid) {
		$tb = TB_WEB_CACHE;
		$q = <<<EOS
			select a.acc_pin as login,
				   u.userfio as fio,
				   u.rolename as role,
				   g.usergroupname as rolename
			from $tb a
			inner join tbusers u on (a.acc_pin = u.userlogin)
			inner join tbusergroups g on (u.rolename = g.rolename)
			where a.acc_userid = '$webid'
EOS;
		return $this->getQueryResultAsArray($q, true);
	}

	/**
	 * get_account_by_auth
	 */
	function get_account_by_auth($login, $pwd) {
		$q = <<<EOS
			select u.userlogin as login,
				   u.userfio as fio,
				   u.rolename as role,
				   g.usergroupname as rolename
			from tbusers u
			inner join tbusergroups g on (u.rolename = g.rolename)
			where u.userlogin = '$login' and u.pwd = '$pwd'
EOS;
		return $this->getQueryResultAsArray($q, true);
	}

	/**
	 * status_task
	 */
	function status_task($taskid) {
		$q = <<<EOS
			select tm_task_id,
				   tm_task_project_id,
				   tm_task_status,
				   tm_task_pause_till,
				   tm_task_dial_num,
				   tm_task_try_count,
				   tm_task_auth_id,
				   cast(tm_task_params as varchar(4096)) as tm_task_params,
				   tm_task_result,
				   tm_task_last_time,
				   tm_task_dial_cause
			from tm_task_list
			where tm_task_id = $taskid
EOS;
		return $this->getQueryResultAsArray($q, true);
	}

	/**
	 * get_project_list_personal
	 *
	 * @return array
	 */
	function get_project_list_personal($userrole,
		$page = 1, $resultsperpage = 99999,
		$sortname = null, $sortorder = null,
		$searchname = null, $searchquery = null,
		&$totalcnt) {
		global $session, $logger;

		$logger->log("database->get_project_list_personal($page, $resultsperpage, $sortname, $sortorder, $searchname, '$searchquery')", PEAR_LOG_DEBUG);

		//$starttm = $session->sf_startdate_proj . ' 00:00:00';
		//$endtm = $session->sf_enddate_proj . ' 23:59:59';

		if (!$sortname)
			$sortname = 'creattime';
		if (!$sortorder)
			$sortorder = 'desc';
		$sort = "order by $sortname $sortorder";

		if (!$page)
			$page = 1;
		if (!$resultsperpage)
			$resultsperpage = 30;
		$start = ($page - 1) * $resultsperpage;
		$limit = "first $resultsperpage skip $start";

		//$searchname = $_POST['qtype'];
		//$searchquery = $_POST['query'];
		$addwhere = "";
		if ($searchquery) {
			$addwhere = "and $searchname like ";
			$addwhere .= (substr($searchquery, 0, 1) == '^') ?
					"'" . substr($searchquery, 1) . "%'" :
					"'%$searchquery%'";
		}

		$qcnt = "select count(*) from tm_project_list p "
				. " join tm_project_users u on (u.tm_project_id = p.tm_project_id and u.rolename = '$userrole')";
		$cntres = $this->getQueryResultAsArray($qcnt, true);
		$totalcnt = empty($cntres) ? 0 : $cntres['count'];

		if ($totalcnt > 0) {
			$q = "select $limit * from tm_project_list p "
				 . " join tm_project_users u on (u.tm_project_id = p.tm_project_id and u.rolename = '$userrole')"; // $addwhere $sort
			return $this->getQueryResultAsArray($q);
		} else
			return array();
	}

	/**
	 * get_project_userrights
	 *
	 * @return int
	 */
	function get_project_userrights($project_id, $userrole) {
		$res = $this->getQueryResultAsArray("select project_accsess_mode from tm_project_users where rolename='$userrole' and tm_project_id=$project_id", true);
		return !empty($res) ? intval($res['project_accsess_mode']) : -1;
	}


	/**
	 * get_task_userrights
	 *
	 * @return int
	 */
	function get_task_userrights($task_id, $userrole) {
		$res = $this->getQueryResultAsArray("select project_accsess_mode from tm_project_users where rolename='$userrole'"
			." and tm_project_id=(select tm_task_project_id from tm_task_list where tm_task_id=$task_id)", true);
		return !empty($res) ? intval($res['project_accsess_mode']) : -1;
	}

	/**
	 * get_project_snapshot_list
	 *
	 * @return array
	 */
	function get_project_snapshot_list($project_id) {
		$project_history = $this->getQueryResultAsArray("select * from tm_project_history where tm_project_id=$project_id order by tm_project_activate asc");
		$res = array();
		foreach ($project_history as $snapshot) {
			$dt = date_parse($snapshot['tm_project_activate']);
			$dta = sprintf("%02d.%02d.%04d %02d:%02d:%02d",
				$dt['day'], $dt['month'], $dt['year'], $dt['hour'], $dt['minute'], $dt['second']);

			if (!empty($snapshot['tm_project_deactivate'])) {
				$dt = date_parse($snapshot['tm_project_deactivate']);
				$dtd = sprintf("%02d.%02d.%04d %02d:%02d:%02d",
					$dt['day'], $dt['month'], $dt['year'], $dt['hour'], $dt['minute'], $dt['second']);
			} else
				$dtd = '';

			$res[] = array(
				'id' => $dta,
				'name' => $dta . ($dtd ? ' - ' . $dtd : ' - по настоящее время')
			);
		}
		$res[] = array('id' => '0', 'name' => 'Пока не выполнявшиеся задачи');
		return $res;
	}

	/**
	 * get_project_snapshot_params
	 *
	 * @return array
	 */
	function get_project_snapshot_params($project_id, $snapshot_date) {
		$result = array('is_complete' => false);

		$snap = $snapshot_date == '0' ? null :
			$this->getQueryResultAsArray("select * from tm_project_history where tm_project_id=$project_id and "
				. "tm_project_activate between '$snapshot_date' and cast('$snapshot_date' as timestamp) + 1/86400", true);

		if (!empty($snap)) {
			$dt = date_parse($snap['tm_project_activate']);
			$result['starttm'] = sprintf("%02d.%02d.%04d %02d:%02d:%02d",
				$dt['day'], $dt['month'], $dt['year'], $dt['hour'], $dt['minute'], $dt['second']);
			if (!empty($snap['tm_project_deactivate'])) {
				$dt = date_parse($snap['tm_project_deactivate']);
				$result['endtm'] = sprintf("%02d.%02d.%04d %02d:%02d:%02d",
					$dt['day'], $dt['month'], $dt['year'], $dt['hour'], $dt['minute'], $dt['second']);
				$result['is_complete'] = true;
			} else {
				$result['endtm'] = date("d.m.Y") . ' 23:59:59'; //null show future tasks?
			}
		} else {
			$result['starttm'] = date("d.m.Y", mktime(0, 0, 0, date("m"), date("d"), date("Y") + 1)) . ' 00:00:00';
			$result['endtm'] = date("d.m.Y", mktime(0, 0, 0, date("m"), date("d"), date("Y") + 2)) . ' 23:59:59';
		}

		return $result;
	}

	/**
	 * get_project_params
	 *
	 * @return array
	 */
	function get_project_params($project_id) {
		$res = $this->getQueryResultAsArray("select * from tm_project_list where tm_project_id=$project_id", true);
		return !empty($res) ? $res : array('tm_project_name' => 'unknown', 'tm_project_menuid' => -1);
	}

	/**
	 * set_project_param
	 *
	 * @return int
	 */
	function set_project_param($project_id, $pname, $pvalue, $ptype = 'integer') {
		global $logger;

		$q = "update tm_project_list set $pname=? where tm_project_id=$project_id";
		$pq = $this->prepare($q, array($ptype), MDB2_PREPARE_MANIP);
		$result = $this->execute($pq, array($pvalue));
		if (PEAR::isError($result) || !is_numeric($result)) {
			$logger->log("database->set_project_param($project_id, $pname, $pvalue):ERR=" . getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		}
		return $result;
	}

	/**
	 * set_task_param
	 *
	 * @return int
	 */
	function set_task_param($task_id, $pname, $pvalue, $ptype = 'integer') {
		global $logger;

		$q = "update tm_task_list set $pname=? where tm_task_id=$task_id";
		$pq = $this->prepare($q, array($ptype), MDB2_PREPARE_MANIP);
		$result = $this->execute($pq, array($pvalue));
		if (PEAR::isError($result) || !is_numeric($result)) {
			$logger->log("database->set_task_param($task_id, $pname, $pvalue):ERR=" . getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		}
		return $result;
	}

	/**
	 * update_project_task
	 *
	 * @return integer
	 */
	function update_project_task($proj_id, $d_arr, &$flagsout) {
		global $logger;

		//$logger->log("update_project_task(ENC): " . mb_detect_encoding($d_arr[7], 'UTF-8, Windows-1251, CP866, KOI8-R', true), PEAR_LOG_DEBUG);

		$tm_task_status = 102;
		if (isset($d_arr[0]) and $d_arr[0] !== '') $tm_task_status = $d_arr[0];
		$tm_task_till_date = 'null';
		if (isset($d_arr[1]) and $d_arr[1] !== '') $tm_task_till_date = "'" . $d_arr[1] . "'";
		$tm_task_dial_num = 'null';
		if (isset($d_arr[2]) and $d_arr[2] !== '') $tm_task_dial_num = "'" . $d_arr[2] . "'";
		$tm_task_try_count = 1;
		if (isset($d_arr[3]) and $d_arr[3] !== '') $tm_task_try_count = $d_arr[3];
		$tm_task_auth_id = 'null';
		if (isset($d_arr[4]) and $d_arr[4] !== '') $tm_task_auth_id = "'" . $d_arr[4] . "'";
		$tm_task_params = 'null';
		if (isset($d_arr[5]) and $d_arr[5] !== '') $tm_task_params = "'" . utf8_encode_safe($d_arr[5]) . "'";
		$tm_task_result = 'null';
		if (isset($d_arr[6]) and $d_arr[6] !== '') $tm_task_result = $d_arr[6];
		$tm_task_ext_id = 'null';
		if (isset($d_arr[7]) and $d_arr[7] !== '') $tm_task_ext_id = "'" . utf8_encode_safe($d_arr[7]) . "'";
		$q = <<<EOS
			select * from tm_init_project($proj_id,
				$tm_task_status,
				$tm_task_till_date,
				$tm_task_dial_num,
				$tm_task_try_count,
				$tm_task_auth_id,
				$tm_task_params,
				$tm_task_result,
				$tm_task_ext_id)
EOS;
		$res = $this->getQueryResultAsArray($q, true);
		$logger->log("update_project_task($q) RES:\n" . print_r($res, true), PEAR_LOG_DEBUG);
		$s_result = null;
		if (!empty($res)) {
			$s_result = $res['tm_task_id'];
			$flagsout = $res['flagsout'];
		}
		return $s_result;
	}

	/**
	 * get_project_task_list
	 *
	 * @return array
	 */
	function get_project_task_list($project_id,
		$starttm, $endtm,
		$flagsin,
		$page = 1, $resultsperpage = 99999,
		$sortname = null, $sortorder = null,
		$searchname = null, $searchquery = null,
		&$totalcnt) {
		global $session, $logger;

		$logger->log("database->get_project_task_list($project_id, $starttm, $endtm, $flagsin, $page, $resultsperpage, $sortname, $sortorder, $searchname, '$searchquery')", PEAR_LOG_DEBUG);

		//$starttm = $session->sf_startdate_proj . ' 00:00:00';
		//$endtm = $session->sf_enddate_proj . ' 23:59:59';
		$starttm = (isset($starttm) && !empty($starttm)) ? "'$starttm'" : 'null';
		$endtm = (isset($endtm) && !empty($endtm)) ? "'$endtm'" : 'null';

		if (!isset($flagsin) || $flagsin === '')
			$flagsin = 'null';

		if (!$sortname)
			$sortname = 'tm_task_id';
		if (!$sortorder)
			$sortorder = 'desc';
		$sort = "order by $sortname $sortorder";

		if (!$page)
			$page = 1;
		if (!$resultsperpage)
			$resultsperpage = 30;
		$start = ($page - 1) * $resultsperpage;
		$limit = "first $resultsperpage skip $start";

		//$searchname = $_POST['qtype'];
		//$searchquery = $_POST['query'];
		$addwhere = "";
		if ($searchquery) {
			$addwhere = "and $searchname like ";
			$addwhere .= (substr($searchquery, 0, 1) == '^') ?
					"'" . substr($searchquery, 1) . "%'" :
					"'%$searchquery%'";
		}

		$qcnt = "select count(*) from tm_get_project_detail($project_id, $starttm, $endtm, $flagsin)";
		$cntres = $this->getQueryResultAsArray($qcnt, true);
		$totalcnt = empty($cntres) ? 0 : $cntres['count'];

		if ($totalcnt > 0) {
			$q = "select $limit * from tm_get_project_detail($project_id, $starttm, $endtm, $flagsin)"; // $addwhere $sort
			return $this->getQueryResultAsArray($q);
		} else
			return array();
	}

	/**
	 * get_task_params
	 *
	 * @return array
	 */
	function get_task_params($task_id) {
		$q = <<<EOS
			select tm_task_id,
				   tm_task_project_id,
				   tm_task_status,
				   tm_task_pause_till,
				   tm_task_dial_num,
				   tm_task_auth_id,
				   cast(tm_task_params as varchar(4096)) as tm_task_params
			from tm_task_list
			where tm_task_id = $task_id
EOS;
		return $this->getQueryResultAsArray($q, true);
	}

	/**
	 * get_task_stats
	 *
	 * @return array
	 */
	function get_task_stats($task_id,
		$page = 1, $resultsperpage = 99999,
		$sortname = null, $sortorder = null,
		$searchname = null, $searchquery = null,
		&$totalcnt) {
		global $session, $logger;

		$logger->log("database->get_task_stats($task_id, $page, $resultsperpage, $sortname, $sortorder, $searchname, '$searchquery')", PEAR_LOG_DEBUG);

		//$starttm = $session->sf_startdate_proj . ' 00:00:00';
		//$endtm = $session->sf_enddate_proj . ' 23:59:59';

		if (!$sortname)
			$sortname = 'tm_task_start_time';
		if (!$sortorder)
			$sortorder = 'desc';
		$sort = "order by $sortname $sortorder";

		if (!$page)
			$page = 1;
		if (!$resultsperpage)
			$resultsperpage = 500;
		$start = ($page - 1) * $resultsperpage;
		$limit = "first $resultsperpage skip $start";

		//$searchname = $_POST['qtype'];
		//$searchquery = $_POST['query'];
		$addwhere = "";
		if ($searchquery) {
			$addwhere = "and $searchname like ";
			$addwhere .= (substr($searchquery, 0, 1) == '^') ?
					"'" . substr($searchquery, 1) . "%'" :
					"'%$searchquery%'";
		}

		$qcnt = "select count(*) from tm_get_task_detail($task_id)";
		$cntres = $this->getQueryResultAsArray($qcnt, true);
		$totalcnt = empty($cntres) ? 0 : $cntres['count'];

		if ($totalcnt > 0) {
			$q = "select $limit * from tm_get_task_detail($task_id)"; // $addwhere $sort
			return $this->getQueryResultAsArray($q);
		} else
			return array();
	}

	/**
	 * get_project_tasks_info
	 */
	function get_project_tasks_info($projectid, $result_status) {
		//global $logger;

		//$logger->log("database->get_project_tasks_info($projectid)", PEAR_LOG_DEBUG);

		$q = "select * from tm_get_project_info($projectid, $result_status)";
		return $this->getQueryResultAsArray($q, true);
	}

	/**
	 * project_createsnapshot
	 */
	/*function project_createsnapshot($userrole, $projectid) {
		global $logger;

		$logger->log("database->project_createsnapshot($userrole, $projectid)", PEAR_LOG_DEBUG);

		$q = <<<EOS
insert into tm_project_list
select (select max(tm_project_id) + 1
		from tm_project_list),
		tm_project_task_event,
		tm_project_graphic,
		tm_project_name || ' #' || cast('Now' as date),
		tm_project_start_time,
		tm_project_stop_time,
		tm_project_noanswer_to,
		tm_project_busy_to,
		tm_project_answer_to,
		tm_project_syserr_to,
		tm_project_task_max,
		tm_project_auth_type,
		tm_project_auth_id,
		tm_project_dial_count,
		tm_project_old_event,
		tm_project_default_voice,
		tm_project_menuid
from tm_project_list
where tm_project_id = $projectid
EOS;
		$pq = $this->prepare($q, null, MDB2_PREPARE_MANIP);
		$result = $this->execute($pq);
		if (PEAR::isError($result) || !is_numeric($result)) {
			$logger->log("database->project_activate($q):ERR1=\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		}
		if ($result == 1) {
			$q = "update tm_project_list set tm_project_old_event = null where tm_project_id = $projectid";
			$pq = $this->prepare($q, null, MDB2_PREPARE_MANIP);
			$result = $this->execute($pq);
			if (PEAR::isError($result) || !is_numeric($result)) {
				$logger->log("database->project_activate($q):ERR2=\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
				return false;
			} else
				$logger->log("database->project_activate():OK=activated", PEAR_LOG_DEBUG);
		} /*else
			$logger->log("database->project_activate():OK=noactivation", PEAR_LOG_DEBUG);* /

		return false;
	}*/

	/**
	 * project_activate
	 */
	function project_activate($userrole, $projectid, $activate) {
		global $logger;

		//$logger->log("database->project_activate($userrole, $projectid, $activate)", PEAR_LOG_DEBUG);

		if ($activate) {
			$q = <<<EOS
				update tm_project_list p
				set p.tm_project_task_event = (select e.tm_event_id
											   from tm_event_list e
											   where e.tm_event_id = p.tm_project_old_event)
				where p.tm_project_task_event is null and
					  p.tm_project_old_event is not null and
					  p.tm_project_id in (select u.tm_project_id
										  from tm_project_users u
										  where u.tm_project_id = $projectid and
												u.rolename = '$userrole')
EOS;
			$pq = $this->prepare($q, null, MDB2_PREPARE_MANIP);
			$result = $this->execute($pq);
			if (PEAR::isError($result) || !is_numeric($result)) {
				$logger->log("database->project_activate($q):ERR1=\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
				return false;
			}
			if ($result == 1) {
				$q = "update tm_project_list set tm_project_old_event = null where tm_project_id = $projectid";
				$pq = $this->prepare($q, null, MDB2_PREPARE_MANIP);
				$result = $this->execute($pq);
				if (PEAR::isError($result) || !is_numeric($result)) {
					$logger->log("database->project_activate($q):ERR2=\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
					return false;
				} else
					$logger->log("database->project_activate():OK=activated", PEAR_LOG_DEBUG);
			} /*else
				$logger->log("database->project_activate():OK=noactivation", PEAR_LOG_DEBUG);*/
		} else {
			$q = <<<EOS
				update tm_project_list p
				set p.tm_project_old_event = p.tm_project_task_event
				where p.tm_project_task_event is not null and
					  p.tm_project_old_event is null and
					  p.tm_project_id in (select u.tm_project_id
										  from tm_project_users u
										  where u.tm_project_id = $projectid and
												u.rolename = '$userrole')
EOS;
			$pq = $this->prepare($q, null, MDB2_PREPARE_MANIP);
			$result = $this->execute($pq);
			if (PEAR::isError($result) || !is_numeric($result)) {
				$logger->log("database->project_activate($q):ERR1=\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
				return false;
			}
			if ($result == 1) {
				$q = "update tm_project_list set tm_project_task_event = null where tm_project_id = $projectid";
				$pq = $this->prepare($q, null, MDB2_PREPARE_MANIP);
				$result = $this->execute($pq);
				if (PEAR::isError($result) || !is_numeric($result)) {
					$logger->log("database->project_activate($q):ERR2=\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
					return false;
				} else
					$logger->log("database->project_activate():OK=deactivated", PEAR_LOG_DEBUG);
			} /*else
				$logger->log("database->project_activate():OK=noactivation", PEAR_LOG_DEBUG);*/
		}

		return true;
	}

	/**
	 * task_activate
	 */
	function task_activate($userrole, $taskid, $activate) {
		global $logger;

		if ($activate) {
			$q = "update tm_task_list set tm_task_try_count = 0 where tm_task_id = $taskid";
			$pq = $this->prepare($q, null, MDB2_PREPARE_MANIP);
			$result = $this->execute($pq);
			if (PEAR::isError($result)) {
				$logger->log("database->task_activate($q)(trycount):ERR=\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
				return false;
			}
		}

		$newtaskstatus = $activate ? TMS_STATUS_READY_FOR_DIAL : TMS_STATUS_STOP_BLOCKING;
		$q = "update tm_task_list set tm_task_status = $newtaskstatus where tm_task_id = $taskid";
		$pq = $this->prepare($q, null, MDB2_PREPARE_MANIP);
		$result = $this->execute($pq);
		if (PEAR::isError($result)) {
			$logger->log("database->task_activate($q):ERR=\n" . getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		} else
			return $result > 0;
	}

	/**
	 * get_project_billing_account
	 *
	 * @return bool|int|array
	 *
	 * returns:
	 * false if billing not supported
	 * -1 if billing account not found
	 * array(accid, emitkcid, number, pin) if billing available
	 */
	function get_project_billing_account($project_id) {
		$project_params = $this->getQueryResultAsArray(
			"select tm_project_auth_type, tm_project_auth_id from tm_project_list where tm_project_id=$project_id", true);
		if (empty($project_params) || !in_array($project_params['tm_project_auth_type'], array(1, 2)))
			return false;

		switch ($project_params['tm_project_auth_type']) {
			case 1:
				$dbres = $this->getQueryResultAsArray(
					//"select accountid, emitterkcid from tbphones where numbera='{$project_params['tm_project_auth_id']}' and authorizationtypeid=0"
					<<<EOS
select ac.emitterkcid,
	   ac.accountid,
	   ac.createdate,
	   ac.fio,
	   ac.currencyid,
	   ac.maxconnections,
	   cu.currencyname,
	   au.summ,
	   '' as pin,
	   ph.numbera as number
from tbaccountsupdate au
inner join tbaccounts ac on (au.emitterkcid = ac.emitterkcid) and (au.accountid = ac.accountid)
inner join tbcurrency cu on (ac.currencyid = cu.currencyid)
inner join tbphones ph on (ac.emitterkcid = ph.emitterkcid) and (ac.accountid = ph.accountid)
where ph.numbera = '{$project_params['tm_project_auth_id']}' and ph.authorizationtypeid = 0
EOS
					, true);
				return (empty($dbres) || !is_numeric($dbres['accountid'])) ? -1 : $dbres;
				break;

			case 2:
				$dbres = $this->getQueryResultAsArray(
					//"select accountid, emitterkcid from tbpins where pin='{$project_params['tm_project_auth_id']}'"
					<<<EOS
select ac.emitterkcid,
	ac.accountid,
	ac.createdate,
	ac.fio,
	ac.currencyid,
	ac.maxconnections,
	cu.currencyname,
	au.summ,
	p.pin,
	'' as number
from tbaccountsupdate au
inner join tbaccounts ac on (au.emitterkcid = ac.emitterkcid) and (au.accountid = ac.accountid)
inner join tbcurrency cu on (ac.currencyid = cu.currencyid)
inner join tbpins p on (ac.emitterkcid = p.emitterkcid) and (ac.accountid = p.accountid)
where p.pin = '{$project_params['tm_project_auth_id']}'
EOS
					, true);
				return (empty($dbres) || !is_numeric($dbres['accountid'])) ? -1 : $dbres;
				break;
		}
		return false;
	}

	/**
	 * get_project_stats_calls
	 */
	function get_project_stats_calls($project_id,
			$flagsin = -1,
			$page = 1, $resultsperpage = 50,
			$sortname = null, $sortorder = null,
			$searchname = null, $searchquery = null,
			&$totalcnt) {
		global $session, $logger;

		$project_billing_account = $this->get_project_billing_account($project_id);
		if ($project_billing_account == false || $project_billing_account == -1) {
			$totalcnt = 0;
			return array();
		}

		$number = $project_billing_account['number'];
		$pin = empty($number) ? "'{$project_billing_account['number']}'" : 'null';
		$startnum = 0;
		$endnum = -1;

		$starttm = date("d.m.Y", mktime(0, 0, 0, date("m") - 6, 1, date("Y"))) . ' 00:00:00';
		$endtm = date("d.m.Y") . ' 23:59:59';

		if (!$sortname)
			$sortname = 'creattime, sessionid';
		if (!$sortorder)
			$sortorder = 'desc';
		$sort = "order by $sortname $sortorder";

		if (!$page)
			$page = 1;
		if (!$resultsperpage)
			$resultsperpage = 50;
		$start = ($page - 1) * $resultsperpage;
		$limit = "first $resultsperpage skip $start";

		$addwhere = "";
		if ($searchquery) {
			$addwhere = "and $searchname like ";
			$addwhere .= (substr($searchquery, 0, 1) == '^') ?
					"'". substr($searchquery, 1) . "%'" :
					"'%$searchquery%'";
		}

		$where = '';
		/*$where = sprintf(" where (realvolume is null or realvolume >= %d and realvolume <= %d) ", 60 * $session->sf_startvol, 60 * $session->sf_endvol);
		if (!empty($session->sf_numbera))
			$where .= sprintf(" and numbera like '%%'||'%s'||'%%' ", $session->sf_numbera);
		if (!empty($session->sf_numberb))
			$where .= sprintf(" and numberb like '%%'||'%s'||'%%' ", $session->sf_numberb);
		if (!empty($session->sf_geo))
			$where .= sprintf(" and geographyname like '%%'||'%s'||'%%' ", strtoupper($session->sf_geo));
		if ($session->sf_calltype100 != '1')
			$where .= " and calltype <> 100 ";
		if ($session->sf_calltype101 != '1')
			$where .= " and calltype <> 101 ";
		if ($session->sf_calltype301 != '1')
			$where .= " and calltype <> 301 ";*/

		$qcnt = "select count(creattime) from int_reportaccountmtr('$number', $pin, '$starttm', '$endtm', $startnum, $endnum, $flagsin) $where $addwhere";
		$cntres = $this->getQueryResultAsArray($qcnt, true);
		$totalcnt = empty($cntres) ? 0 : $cntres['count'];
		if ($totalcnt > 0) {
			$q = "select $limit * from int_reportaccountmtr('$number', $pin, '$starttm', '$endtm', $startnum, $endnum, $flagsin) $where $addwhere $sort";
			return $this->getQueryResultAsArray($q);
		} else
			return array();
	}

	/**
	 * get_project_stats_pays
	 */
	function get_project_stats_pays($project_id,
			$page = 1, $resultsperpage = 99999,
			$sortname = null, $sortorder = null,
			$searchname = null, $searchquery = null,
			&$totalcnt) {
		global $session, $logger;

		//$logger->log("database->get_project_stats_pays($project_id, $page, $resultsperpage, $sortname, $sortorder, $searchname, '$searchquery')", PEAR_LOG_DEBUG);

		$project_billing_account = $this->get_project_billing_account($project_id);
		if ($project_billing_account == false || $project_billing_account == -1) {
			$totalcnt = 0;
			return array();
		}

		$starttm = date("d.m.Y", mktime(0, 0, 0, date("m") - 6, 1, date("Y"))) . ' 00:00:00';
		$endtm = date("d.m.Y") . ' 23:59:59';

		if (!$sortname)
			$sortname = 'paydate';
		if (!$sortorder)
			$sortorder = 'desc';
		$sort = "order by $sortname $sortorder";

		if (!$page)
			$page = 1;
		if (!$resultsperpage)
			$resultsperpage = 30;
		$start = ($page - 1) * $resultsperpage;
		$limit = "first $resultsperpage skip $start";

		$searchname = $_POST['qtype'];
		$searchquery = $_POST['query'];
		$addwhere = "";
		if ($searchquery) {
			$addwhere = "and $searchname like ";
			$addwhere .= (substr($searchquery, 0, 1) == '^') ?
					"'". substr($searchquery, 1) . "%'" :
					"'%$searchquery%'";
		}

		$q = <<<EOS
			select count(a.adddate)
			from tbaddsumm a
			left outer join tbcurrency b on (a.currencyid = b.currencyid)
			where a.emitterkcid = {$project_billing_account['emitterkcid']} and a.accountid = {$project_billing_account['accountid']} and
				  a.adddate >= '$starttm' and a.adddate <= '$endtm'
				  $addwhere
EOS;
		$cntres = $this->getQueryResultAsArray($q, true);
		$totalcnt = empty($cntres) ? 0 : $cntres['count'];
		if ($totalcnt > 0) {
			$q = <<<EOS
				select $limit a.adddate as paydate,
					   round(a.summ, 2) as paysumm,
					   b.currencyname as paycurrency,
					   a.addinfo as payinfo,
					   a.addby as paysrc
				from tbaddsumm a
				left outer join tbcurrency b on (a.currencyid = b.currencyid)
				where a.emitterkcid = {$project_billing_account['emitterkcid']} and a.accountid = {$project_billing_account['accountid']} and
					  a.adddate >= '$starttm' and a.adddate <= '$endtm'
					  $addwhere $sort
EOS;
			return $this->getQueryResultAsArray($q);
		} else
			return array();
	}

};

$database = new TMDB;

?>
