<?php
/**
 * Form.php
 */
require_once(dirname(__FILE__) . '/session.php');

class Form {
	public $values = array();
	public $errors = array();
	public $num_errors;

	function Form() {
		/**
		 * Get form value and error arrays, used when there
		 * is an error with a user-submitted form.
		 */
		global $session;

		if (isset($session->value_array) && isset($session->error_array)) {
			$this->values = $session->value_array;
			$this->errors = $session->error_array;
			$this->num_errors = count($this->errors);

			unset($session->value_array);
			unset($session->error_array);
		} else {
			$this->num_errors = 0;
		}
	}

	/**
	* setValue - Records the value typed into the given
	* form field by the user.
	*/
	function setValue($field, $value){
			$this->values[$field] = $value;
	}

	/**
	* setError - Records new form error given the form
	* field name and the error message attached to it.
	*/
	function setError($field, $errmsg){
			$this->errors[$field] = $errmsg;
			$this->num_errors = count($this->errors);
	}

	/**
	* value - Returns the value attached to the given
	* field, if none exists, the empty string is returned.
	*/
	function value($field){
			if(array_key_exists($field,$this->values)){
				return htmlspecialchars(stripslashes($this->values[$field]));
			}else{
				return "";
			}
	}

	/**
	* error - Returns the error message attached to the
	* given field, if none exists, the empty string is returned.
	*/
	function error($field){
			if(array_key_exists($field,$this->errors)){
				return "<font color=\"#ff0000\">".$this->errors[$field]."</font>";
			}else{
				return "";
			}
	}

	/**
	* getErrorArray - Returns the array of error messages
	*/
	function getErrorArray(){
			return $this->errors;
	}
};

?>
