<?php
/**
 * Mailer.php
 *
 * The Mailer class is meant to simplify the task of sending
 * emails to users. Note: this email system will not work
 * if your server is not setup to send mail.
 *
 * If you are running Windows and want a mail server, check
 * out this website to see a list of freeware programs:
 * <http://www.snapfiles.com/freeware/server/fwmailserver.html>
 *
 */

class Mailer {
	/**
	 * sendFiles - Sends fax or voice record files
	 * to supplied email address.
	 * @link http://ru.php.net/manual/en/function.mail.php#83491
	 * @param array $files <p>
	 * array of files to attach.
	 *      systemname => '/tmp/aout.tmp',
	 *      displayname => 'voicemessage.mp3',
	 *      type => 'audio/mpeg',
	 *      size => 0,
	 * </p>
	 */
	function sendFiles($to, $subject, $comment, $files) {
		require_once('Mail.php');
		require_once('Mail/mime.php');
		require_once(dirname(__FILE__).'/constants.php');

		global $logger;
		$logger->log('sendFiles:'.print_r($files, true));

		$To = strip_tags($to);
		$TextMessage = strip_tags(nl2br($comment), "<br>");
		$HTMLMessage = nl2br($comment);
		$FromName = strip_tags(EMAIL_FROM_NAME);
		$FromEmail = strip_tags(EMAIL_FROM_ADDR);
		$Subject = strip_tags($subject);

		$hdrs = array(
			'From' => "$FromName <$FromEmail>",
			'Reply-To' => $FromEmail,
			'Subject' => $Subject,
			'User-Agent' => 'RINO Mailer 20100629',
			'X-Mailer' => 'RINO Mailer 20100629'
		);

		$mime = new Mail_mime("\n");
		$mime->setTXTBody($TextMessage);
		$mime->setHTMLBody($HTMLMessage);
		foreach ($files as $file)
			$mime->addAttachment($file['systemname'], $file['type'], $file['displayname'],
				true, 'base64', 'attachment', 'windows-1251');
		$body = $mime->get(array(
			'head_charset' => 'windows-1251',
			//'text_encoding' => '8bit',
			//'html_encoding' => '8bit',
			'text_charset' => 'windows-1251',
			'html_charset' => 'windows-1251'));
		$hdrs = $mime->headers($hdrs);
		$mail =& Mail::factory('mail');
		//$logger->log('sendFiles(body):'.print_r($body, true));
		$res = $mail->send($To, $hdrs, $body);
		if (PEAR::isError($res)) {
			//$logger->log("sendFiles() ERR: ".print_r($res, true));
			$logger->log("sendFiles() ERR: ".$res->message);
			return false;
		}
		$logger->log('sendFiles(res):'.print_r($res, true));
		return $res;
	}

	/**
	 * sendFilesOld - Sends fax or voice record files
	 * to supplied email address.
	 * @link http://ru.php.net/manual/en/function.mail.php#83491
	 * @param array $files <p>
	 * array of files to attach.
	 *      systemname => '/tmp/aout.tmp',
	 *      displayname => 'voicemessage.mp3',
	 *      type => 'audio/mpeg',
	 *      size => 0,
	 * </p>
	 */
	function sendFilesOld($to, $subject, $comment, $files) {
		require_once(dirname(__FILE__) . '/constants.php');
		global $logger;

		$logger->log('sendFiles:'.print_r($files, true));

		$To = strip_tags($to);
		$TextMessage = strip_tags(nl2br($comment), "<br>");
		$HTMLMessage = nl2br($comment);
		$FromName = strip_tags(EMAIL_FROM_NAME);
		$FromEmail = strip_tags(EMAIL_FROM_ADDR);
		$Subject = strip_tags($subject);

		$boundary1 = rand(0,9)."-".rand(10000000000,9999999999)."-".rand(10000000000,9999999999)."=:".rand(10000,99999);
		$boundary2 = rand(0,9)."-".rand(10000000000,9999999999)."-".rand(10000000000,9999999999)."=:".rand(10000,99999);

		foreach ($files as $file) {
			if (!empty($file['size']) && !empty($file['systemname'])) {
				$attach = 'yes';

				$fh = fopen($file['systemname'], 'rb');
				$f_contents = fread($fh, $file['size']);
				fclose($fh);

				$attachment[] = chunk_split(base64_encode($f_contents));
				$ftype[] = $file['type'];
				$fname[] = $file['displayname'];
			}
		}

		if ($attach !== 'yes') {
			$Headers = <<<EOS
From: $FromName <$FromEmail>
Reply-To: $FromEmail
MIME-Version: 1.0
Content-Type: multipart/alternative;
	boundary="$boundary1"
EOS;

			$Body = <<<EOS
MIME-Version: 1.0
Content-Type: multipart/alternative;
	boundary="$boundary1"

This is a multi-part message in MIME format.

--$boundary1
Content-Type: text/plain;
	charset="windows-1251"
Content-Transfer-Encoding: 8bit

$TextMessage
--$boundary1
Content-Type: text/html;
	charset="windows-1251"
Content-Transfer-Encoding: 8bit

$HTMLMessage

--$boundary1--
EOS;
		} else {
			$attachments = '';
			$Headers = <<<EOS
From: $FromName <$FromEmail>
Reply-To: $FromEmail
MIME-Version: 1.0
Content-Type: multipart/mixed;
	boundary="$boundary1"
EOS;

			for($j = 0; $j < count($ftype); $j++) {
				$attachments .= <<<EOS
--$boundary1
Content-Type: $ftype[$j];
	name="$fname[$j]"
Content-Transfer-Encoding: base64
Content-Disposition: attachment;
	filename="$fname[$j]"

$attachment[$j]

EOS;
			}

			$Body = <<<EOS
This is a multi-part message in MIME format.

--$boundary1
Content-Type: multipart/alternative;
	boundary="$boundary2"

--$boundary2
Content-Type: text/plain;
	charset="windows-1251"
Content-Transfer-Encoding: 8bit

$TextMessage
--$boundary2
Content-Type: text/html;
	charset="windows-1251"
Content-Transfer-Encoding: 8bit

$HTMLMessage

--$boundary2--

$attachments
--$boundary1--
EOS;
		}

		$logger->log('sendFiles(body):'.print_r($Body, true));
		$res = mail($To, $Subject, $Body, $Headers);
		$logger->log('sendFiles(res):'.print_r($res, true));
		return $res;
	}

	/**
	* sendWelcome - Sends a welcome message to the newly
	* registered user, also supplying the username and
	* password.
	*/
	function sendWelcome($user, $email, $pass) {
		$from = "From: ".EMAIL_FROM_NAME." <".EMAIL_FROM_ADDR.">";
		$subject = "Welcome!";
		$body = $user.",\n\n"
					 ."Welcome! You've just registered at our Site "
					 ."with the following information:\n\n"
					 ."Username: ".$user."\n"
					 ."Password: ".$pass."\n\n"
					 ."If you ever lose or forget your password, a new "
					 ."password will be generated for you and sent to this "
					 ."email address, if you would like to change your "
					 ."email address you can do so by going to the "
					 ."My Account page after signing in.\n\n"
					 ."- Our Site";

		return mail($email,$subject,$body,$from);
	}

	/**
	* sendNewPass - Sends the newly generated password
	* to the user's email address that was specified at
	* sign-up.
	*/
	function sendNewPass($user, $email, $pass){
		$from = "From: ".EMAIL_FROM_NAME." <".EMAIL_FROM_ADDR.">";
		$subject = "Our Site - Your new password";
		$body = $user.",\n\n"
					 ."We've generated a new password for you at your "
					 ."request, you can use this new password with your "
					 ."username to log in to our Site.\n\n"
					 ."Username: ".$user."\n"
					 ."New Password: ".$pass."\n\n"
					 ."It is recommended that you change your password "
					 ."to something that is easier to remember, which "
					 ."can be done by going to the My Account page "
					 ."after signing in.\n\n"
					 ."- Our Site";

		return mail($email,$subject,$body,$from);
	}

};

$mailer = new Mailer;

?>
