<?php
/**
 * Session.php
 */
require_once(dirname(__FILE__) . '/functions.php');
require_once(dirname(__FILE__) . '/database.php');
require_once(dirname(__FILE__) . '/database_dynmenu.php');
require_once(dirname(__FILE__) . '/mailer.php');
require_once(dirname(__FILE__) . '/form.php');

class Session {
	public $db_ok;

	public $userwebid;
	public $syssession; //_SESSION proxy

	public $time;
	public $logged_in = false;

	public function __set($name, $value) {
		$this->syssession[$this->userwebid][$name] = $value;
	}

	public function __get($name) {
		if (array_key_exists($name, $this->syssession[$this->userwebid]))
			return $this->syssession[$this->userwebid][$name];

		/*$trace = debug_backtrace();
		trigger_error(
				'Undefined property via __get(): ' . $name .
				' in ' . $trace[0]['file'] .
				' on line ' . $trace[0]['line'],
				E_USER_NOTICE);*/
		return null;
	}

	public function __isset($name) {
		return isset($this->syssession[$this->userwebid][$name]);
	}

	public function __unset($name) {
		unset($this->syssession[$this->userwebid][$name]);
	}

	/* Params setter/getter */
	public function ssnparam($pname, $pvalue = null) {
		if ($pvalue === null)
			return $this->syssession[$this->userwebid][$pname];
		else
			$this->syssession[$this->userwebid][$pname] = $pvalue;
		return true;
	}

	function get_cookie_expire_time($forceexpiry = false) {
		if ($forceexpiry)
			return time() - COOKIE_EXPIRE;
		else
			return isset($_COOKIE[COOKIE_NAME_WEBIDR]) ? (time() + COOKIE_EXPIRE) : 0;
	}

	function Session() {
		$this->time = time();
		$this->startSession();
	}

	/**
	 * startSession
	 */
	function startSession() {
		global $database, $database_dynmenu/*, $logger*/;
        if (isset($_POST['cookie_hash'])) {
            parse_str(base64_decode($_POST['cookie_hash']), $_COOKIE);
            session_id($_COOKIE['vcsession']);
        }

		session_name('vcsession');
		session_start();

		$this->syssession = &$_SESSION;
		$this->db_ok = $database->db_ok && $database_dynmenu->db_ok;
		$this->logged_in = $this->db_ok && $this->login_check();

		if ($this->logged_in) {
			$database->register_login_session($this->accountinfo['login'], $this->userwebid, $this->time);

			//Set referrer page
			//$logger->log('session.url:'.$this->url);
			if (isset($this->url)) {
				if (preg_match('/(fs_(get|put)file)|(process)$/', $this->url))
					$this->referrer = isset($this->ref) ? $this->ref : '/';
				else
					$this->referrer = $this->ref = $this->url;
			} else
				$this->referrer = $this->ref = '/';
			//$logger->log('session.ref:'.$this->referrer);

			//Set current url
			$this->url = $_SERVER['PHP_SELF'];
		}
		//$logger->log('startSession:'.print_r($this, true));
	}

	/**
	 * login_check
	 *
	 * @return bool
	 */
	function login_check() {
		global $database, $logger;

		//$logger->log("login_check(cook):".print_r($_COOKIE, true));
		//$logger->log("login_check(sess):".print_r($this->syssession, true));

		//check if webid exist
		$this->userwebid = isset($_COOKIE[COOKIE_NAME_WEBID]) ? $_COOKIE[COOKIE_NAME_WEBID] : null;
		if (!$this->userwebid) {
			//$logger->log("login_check:nowebid!\n".print_r($_COOKIE, true));
			return false;
		}

		if (!isset($this->syssession[$this->userwebid]) || !is_array($this->syssession[$this->userwebid]))
			$this->syssession[$this->userwebid] = array();

		//check if webid valid
		$this->accountinfo = $database->get_account_by_webid($this->userwebid);
		if (empty($this->accountinfo)) {
			$this->syssession[$this->userwebid] = array();
			unset($this->syssession[$this->userwebid]);
			return false;
		}

		//webid seems valid. now we have accountinfo
		//$this->userwebid = isset($_COOKIE[COOKIE_NAME_WEBID]) ? $_COOKIE[COOKIE_NAME_WEBID] : $this->generateRandID();
		if (!setcookie(COOKIE_NAME_WEBID, $this->userwebid, $this->get_cookie_expire_time(), COOKIE_PATH))
			$logger->log("login_check->fail setcookie ".COOKIE_NAME_WEBID);

		return true;
	}

	/**
	 * login
	 */
	function login($login, $pwd, $remember) {
		global $database, $logger;

		$logger->log("session->login($login, ***, $remember)");

		$acc = $database->get_account_by_auth($login, $pwd);
		if (empty($acc)) {
			$logger->log("session->login($login, ***, $remember)->auth_fail");
			return false;
		}

		$this->userwebid = $this->generateRandID();
		if (!isset($this->syssession[$this->userwebid]) || !is_array($this->syssession[$this->userwebid]))
			$this->syssession[$this->userwebid] = array();

		$this->accountinfo = $acc;

		$database->register_login_session($this->accountinfo['login'], $this->userwebid, $this->time);
		$database->log_login_session($this->accountinfo['login'], $this->userwebid, $this->time);

		if ($remember)
			setcookie(COOKIE_NAME_WEBIDR, 1, time() + COOKIE_EXPIRE, COOKIE_PATH);
		if (!setcookie(COOKIE_NAME_WEBID, $this->userwebid, $this->get_cookie_expire_time(), COOKIE_PATH))
			$logger->log("session->login($login, ***, $remember)->fail setcookie ".COOKIE_NAME_WEBID);

		return true;
	}

	/**
	 * logout
	 */
	function logout() {
		global $database;

		if ($this->logged_in) {
			$database->register_login_session($this->accountinfo['login'], $this->userwebid, -$this->time);
			$database->log_login_session($this->accountinfo['login'], $this->userwebid, -$this->time);
		}

		setcookie(COOKIE_NAME_WEBID, null, $this->get_cookie_expire_time(true), COOKIE_PATH);
		setcookie(COOKIE_NAME_WEBIDR, null, $this->get_cookie_expire_time(true), COOKIE_PATH);

		$this->logged_in = false;

		$this->syssession[$this->userwebid] = array();
		unset($this->syssession[$this->userwebid]);
	}

	/**
	 * generateRandID - Generates a string made up of randomized
	 * letters (lower and upper case) and digits and returns
	 * the sha1 hash of it to be used as a userwebid.
	 */
	function generateRandID($len = 128) {
		return sha1($this->generateRandStr($len));
	}

	/**
	 * generateRandStr - Generates a string made up of randomized
	 * letters (lower and upper case) and digits, the length
	 * is a specified parameter.
	 */
	function generateRandStr($length) {
		$randstr = "";
		for ($i = 0; $i < $length; $i++) {
			$randnum = mt_rand(0, 61);
			if ($randnum < 10) {
				$randstr .= chr($randnum + 48);
			} else if ($randnum < 36) {
				$randstr .= chr($randnum + 55);
			} else {
				$randstr .= chr($randnum + 61);
			}
		}
		return $randstr;
	}

	/**
	 * initStatFilters - initializes statistics page filters with some initial values if needed
	 */
	function initStatFilters() {
		$result = false;
		if (!isset($this->sf_startdate_proj)) {
			$this->sf_startdate_proj = date("d.m.Y", mktime(0, 0, 0, date("m"), 1, date("Y") - 1));
			$this->sf_enddate_proj = date("d.m.Y");
			$result = true;
		}
		/*if (!isset($this->tfilter_days) || !is_numeric($this->tfilter_days)) {
			$this->tfilter_days = 0;
			$result = true;
		}*/
		return $result;
	}
};

$session = new Session;
$form = new Form;

?>
